@@ -27,7 +27,6 @@ class WP_Tax_Query {
 	 * See WP_Tax_Query::__construct() for information on tax query arguments.
 	 *
 	 * @since 3.1.0
-	 * @access public
 	 * @var array
 	 */
 	public $queries = array();
@@ -36,7 +35,6 @@ class WP_Tax_Query {
 	 * The relation between the queries. Can be one of 'AND' or 'OR'.
 	 *
 	 * @since 3.1.0
-	 * @access public
 	 * @var string
 	 */
 	public $relation;
@@ -45,18 +43,17 @@ class WP_Tax_Query {
 	 * Standard response when the query should not return any rows.
 	 *
 	 * @since 3.2.0
-	 *
-	 * @static
-	 * @access private
 	 * @var string
 	 */
-	private static $no_results = array( 'join' => array( '' ), 'where' => array( '0 = 1' ) );
+	private static $no_results = array(
+		'join'  => array( '' ),
+		'where' => array( '0 = 1' ),
+	);
 
 	/**
 	 * A flat list of table aliases used in the JOIN clauses.
 	 *
 	 * @since 4.1.0
-	 * @access protected
 	 * @var array
 	 */
 	protected $table_aliases = array();
@@ -68,7 +65,6 @@ class WP_Tax_Query {
 	 * number of places by WP_Query.
 	 *
 	 * @since 4.1.0
-	 * @access public
 	 * @var array
 	 */
 	public $queried_terms = array();
@@ -77,7 +73,6 @@ class WP_Tax_Query {
 	 * Database table that where the metadata's objects are stored (eg $wpdb->users).
 	 *
 	 * @since 4.1.0
-	 * @access public
 	 * @var string
 	 */
 	public $primary_table;
@@ -86,7 +81,6 @@ class WP_Tax_Query {
 	 * Column in 'primary_table' that represents the ID of the object.
 	 *
 	 * @since 4.1.0
-	 * @access public
 	 * @var string
 	 */
 	public $primary_id_column;
@@ -96,7 +90,6 @@ class WP_Tax_Query {
 	 *
 	 * @since 3.1.0
 	 * @since 4.1.0 Added support for `$operator` 'NOT EXISTS' and 'EXISTS' values.
-	 * @access public
 	 *
 	 * @param array $tax_query {
 	 *     Array of taxonomy query clauses.
@@ -135,7 +128,6 @@ class WP_Tax_Query {
 	 * each first-order clause contains all the necessary keys from `$defaults`.
 	 *
 	 * @since 4.1.0
-	 * @access public
 	 *
 	 * @param array $queries Array of queries clauses.
 	 * @return array Sanitized array of query clauses.
@@ -144,10 +136,10 @@ class WP_Tax_Query {
 		$cleaned_query = array();
 
 		$defaults = array(
-			'taxonomy' => '',
-			'terms' => array(),
-			'field' => 'term_id',
-			'operator' => 'IN',
+			'taxonomy'         => '',
+			'terms'            => array(),
+			'field'            => 'term_id',
+			'operator'         => 'IN',
 			'include_children' => true,
 		);
 
@@ -155,12 +147,12 @@ class WP_Tax_Query {
 			if ( 'relation' === $key ) {
 				$cleaned_query['relation'] = $this->sanitize_relation( $query );
 
-			// First-order clause.
+				// First-order clause.
 			} elseif ( self::is_first_order_clause( $query ) ) {
 
-				$cleaned_clause = array_merge( $defaults, $query );
+				$cleaned_clause          = array_merge( $defaults, $query );
 				$cleaned_clause['terms'] = (array) $cleaned_clause['terms'];
-				$cleaned_query[] = $cleaned_clause;
+				$cleaned_query[]         = $cleaned_clause;
 
 				/*
 				 * Keep a copy of the clause in the flate
@@ -185,7 +177,7 @@ class WP_Tax_Query {
 					}
 				}
 
-			// Otherwise, it's a nested query, so we recurse.
+				// Otherwise, it's a nested query, so we recurse.
 			} elseif ( is_array( $query ) ) {
 				$cleaned_subquery = $this->sanitize_query( $query );
 
@@ -207,7 +199,6 @@ class WP_Tax_Query {
 	 * Sanitize a 'relation' operator.
 	 *
 	 * @since 4.1.0
-	 * @access public
 	 *
 	 * @param string $relation Raw relation key from the query argument.
 	 * @return string Sanitized relation ('AND' or 'OR').
@@ -231,9 +222,6 @@ class WP_Tax_Query {
 	 *
 	 * @since 4.1.0
 	 *
-	 * @static
-	 * @access protected
-	 *
 	 * @param array $query Tax query arguments.
 	 * @return bool Whether the query clause is a first-order clause.
 	 */
@@ -246,9 +234,6 @@ class WP_Tax_Query {
 	 *
 	 * @since 3.1.0
 	 *
-	 * @static
-	 * @access public
-	 *
 	 * @param string $primary_table     Database table where the object being filtered is stored (eg wp_users).
 	 * @param string $primary_id_column ID column for the filtered object in $primary_table.
 	 * @return array {
@@ -259,7 +244,7 @@ class WP_Tax_Query {
 	 * }
 	 */
 	public function get_sql( $primary_table, $primary_id_column ) {
-		$this->primary_table = $primary_table;
+		$this->primary_table     = $primary_table;
 		$this->primary_id_column = $primary_id_column;
 
 		return $this->get_sql_clauses();
@@ -272,7 +257,6 @@ class WP_Tax_Query {
 	 * is abstracted out to maintain parity with the other Query classes.
 	 *
 	 * @since 4.1.0
-	 * @access protected
 	 *
 	 * @return array {
 	 *     Array containing JOIN and WHERE SQL clauses to append to the main query.
@@ -287,7 +271,7 @@ class WP_Tax_Query {
 		 * To keep $this->queries unaltered, pass a copy.
 		 */
 		$queries = $this->queries;
-		$sql = $this->get_sql_for_query( $queries );
+		$sql     = $this->get_sql_for_query( $queries );
 
 		if ( ! empty( $sql['where'] ) ) {
 			$sql['where'] = ' AND ' . $sql['where'];
@@ -303,9 +287,8 @@ class WP_Tax_Query {
 	 * produce the properly nested SQL.
 	 *
 	 * @since 4.1.0
-	 * @access protected
 	 *
-	 * @param array $query Query to parse, passed by reference.
+	 * @param array $query Query to parse (passed by reference).
 	 * @param int   $depth Optional. Number of tree levels deep we currently are.
 	 *                     Used to calculate indentation. Default 0.
 	 * @return array {
@@ -328,7 +311,7 @@ class WP_Tax_Query {
 
 		$indent = '';
 		for ( $i = 0; $i < $depth; $i++ ) {
-			$indent .= "  ";
+			$indent .= '  ';
 		}
 
 		foreach ( $query as $key => &$clause ) {
@@ -350,7 +333,7 @@ class WP_Tax_Query {
 					}
 
 					$sql_chunks['join'] = array_merge( $sql_chunks['join'], $clause_sql['join'] );
-				// This is a subquery, so we recurse.
+					// This is a subquery, so we recurse.
 				} else {
 					$clause_sql = $this->get_sql_for_query( $clause, $depth + 1 );
 
@@ -385,11 +368,10 @@ class WP_Tax_Query {
 	 * Generate SQL JOIN and WHERE clauses for a "first-order" query clause.
 	 *
 	 * @since 4.1.0
-	 * @access public
 	 *
 	 * @global wpdb $wpdb The WordPress database abstraction object.
 	 *
-	 * @param array $clause       Query clause, passed by reference.
+	 * @param array $clause       Query clause (passed by reference).
 	 * @param array $parent_query Parent query array.
 	 * @return array {
 	 *     Array containing JOIN and WHERE SQL clauses to append to a first-order query.
@@ -406,7 +388,8 @@ class WP_Tax_Query {
 			'join'  => array(),
 		);
 
-		$join = $where = '';
+		$join  = '';
+		$where = '';
 
 		$this->clean_query( $clause );
 
@@ -414,7 +397,7 @@ class WP_Tax_Query {
 			return self::$no_results;
 		}
 
-		$terms = $clause['terms'];
+		$terms    = $clause['terms'];
 		$operator = strtoupper( $clause['operator'] );
 
 		if ( 'IN' == $operator ) {
@@ -431,7 +414,7 @@ class WP_Tax_Query {
 			 */
 			$alias = $this->find_compatible_table_alias( $clause, $parent_query );
 			if ( false === $alias ) {
-				$i = count( $this->table_aliases );
+				$i     = count( $this->table_aliases );
 				$alias = $i ? 'tt' . $i : $wpdb->term_relationships;
 
 				// Store the alias as part of a flat array to build future iterators.
@@ -445,7 +428,6 @@ class WP_Tax_Query {
 				$join .= " ON ($this->primary_table.$this->primary_id_column = $alias.object_id)";
 			}
 
-
 			$where = "$alias.term_taxonomy_id $operator ($terms)";
 
 		} elseif ( 'NOT IN' == $operator ) {
@@ -481,14 +463,17 @@ class WP_Tax_Query {
 
 		} elseif ( 'NOT EXISTS' === $operator || 'EXISTS' === $operator ) {
 
-			$where = $wpdb->prepare( "$operator (
+			$where = $wpdb->prepare(
+				"$operator (
 				SELECT 1
 				FROM $wpdb->term_relationships
 				INNER JOIN $wpdb->term_taxonomy
 				ON $wpdb->term_taxonomy.term_taxonomy_id = $wpdb->term_relationships.term_taxonomy_id
 				WHERE $wpdb->term_taxonomy.taxonomy = %s
 				AND $wpdb->term_relationships.object_id = $this->primary_table.$this->primary_id_column
-			)", $clause['taxonomy'] );
+			)",
+				$clause['taxonomy']
+			);
 
 		}
 
@@ -511,7 +496,6 @@ class WP_Tax_Query {
 	 * clauses that are connected by the relation 'OR'.
 	 *
 	 * @since 4.1.0
-	 * @access protected
 	 *
 	 * @param array       $clause       Query clause.
 	 * @param array       $parent_query Parent query of $clause.
@@ -555,7 +539,6 @@ class WP_Tax_Query {
 	 * Validates a single query.
 	 *
 	 * @since 3.2.0
-	 * @access private
 	 *
 	 * @param array $query The single query. Passed by reference.
 	 */
@@ -582,12 +565,13 @@ class WP_Tax_Query {
 		if ( is_taxonomy_hierarchical( $query['taxonomy'] ) && $query['include_children'] ) {
 			$this->transform_query( $query, 'term_id' );
 
-			if ( is_wp_error( $query ) )
+			if ( is_wp_error( $query ) ) {
 				return;
+			}
 
 			$children = array();
 			foreach ( $query['terms'] as $term ) {
-				$children = array_merge( $children, get_term_children( $term, $query['taxonomy'] ) );
+				$children   = array_merge( $children, get_term_children( $term, $query['taxonomy'] ) );
 				$children[] = $term;
 			}
 			$query['terms'] = $children;
@@ -599,6 +583,9 @@ class WP_Tax_Query {
 	/**
 	 * Transforms a single query, from one field to another.
 	 *
+	 * Operates on the `$query` object by reference. In the case of error,
+	 * `$query` is converted to a WP_Error object.
+	 *
 	 * @since 3.2.0
 	 *
 	 * @global wpdb $wpdb The WordPress database abstraction object.
@@ -608,67 +595,62 @@ class WP_Tax_Query {
 	 *                                or 'term_id'. Default 'term_id'.
 	 */
 	public function transform_query( &$query, $resulting_field ) {
-		global $wpdb;
-
-		if ( empty( $query['terms'] ) )
+		if ( empty( $query['terms'] ) ) {
 			return;
+		}
 
-		if ( $query['field'] == $resulting_field )
+		if ( $query['field'] == $resulting_field ) {
 			return;
+		}
 
 		$resulting_field = sanitize_key( $resulting_field );
 
-		switch ( $query['field'] ) {
-			case 'slug':
-			case 'name':
-				foreach ( $query['terms'] as &$term ) {
-					/*
-					 * 0 is the $term_id parameter. We don't have a term ID yet, but it doesn't
-					 * matter because `sanitize_term_field()` ignores the $term_id param when the
-					 * context is 'db'.
-					 */
-					$clean_term = sanitize_term_field( $query['field'], $term, 0, $query['taxonomy'], 'db' );
-
-					// Match sanitization in wp_insert_term().
-					$clean_term = wp_unslash( $clean_term );
-
-					$term = "'" . esc_sql( $clean_term ) . "'";
-				}
-
-				$terms = implode( ",", $query['terms'] );
-
-				$terms = $wpdb->get_col( "
-					SELECT $wpdb->term_taxonomy.$resulting_field
-					FROM $wpdb->term_taxonomy
-					INNER JOIN $wpdb->terms USING (term_id)
-					WHERE taxonomy = '{$query['taxonomy']}'
-					AND $wpdb->terms.{$query['field']} IN ($terms)
-				" );
-				break;
-			case 'term_taxonomy_id':
-				$terms = implode( ',', array_map( 'intval', $query['terms'] ) );
-				$terms = $wpdb->get_col( "
-					SELECT $resulting_field
-					FROM $wpdb->term_taxonomy
-					WHERE term_taxonomy_id IN ($terms)
-				" );
-				break;
-			default:
-				$terms = implode( ',', array_map( 'intval', $query['terms'] ) );
-				$terms = $wpdb->get_col( "
-					SELECT $resulting_field
-					FROM $wpdb->term_taxonomy
-					WHERE taxonomy = '{$query['taxonomy']}'
-					AND term_id IN ($terms)
-				" );
+		// Empty 'terms' always results in a null transformation.
+		$terms = array_filter( $query['terms'] );
+		if ( empty( $terms ) ) {
+			$query['terms'] = array();
+			$query['field'] = $resulting_field;
+			return;
 		}
 
-		if ( 'AND' == $query['operator'] && count( $terms ) < count( $query['terms'] ) ) {
+		$args = array(
+			'get'                    => 'all',
+			'number'                 => 0,
+			'taxonomy'               => $query['taxonomy'],
+			'update_term_meta_cache' => false,
+			'orderby'                => 'none',
+		);
+
+		// Term query parameter name depends on the 'field' being searched on.
+		switch ( $query['field'] ) {
+			case 'slug':
+				$args['slug'] = $terms;
+				break;
+			case 'name':
+				$args['name'] = $terms;
+				break;
+			case 'term_taxonomy_id':
+				$args['term_taxonomy_id'] = $terms;
+				break;
+			default:
+				$args['include'] = wp_parse_id_list( $terms );
+				break;
+		}
+
+		$term_query = new WP_Term_Query();
+		$term_list  = $term_query->query( $args );
+
+		if ( is_wp_error( $term_list ) ) {
+			$query = $term_list;
+			return;
+		}
+
+		if ( 'AND' == $query['operator'] && count( $term_list ) < count( $query['terms'] ) ) {
 			$query = new WP_Error( 'inexistent_terms', __( 'Inexistent terms.' ) );
 			return;
 		}
 
-		$query['terms'] = $terms;
+		$query['terms'] = wp_list_pluck( $term_list, $resulting_field );
 		$query['field'] = $resulting_field;
 	}
 }
